@@ -18,12 +18,12 @@
 #include "env.h"
 #include "hints.h"
 #include "libutil.h"
-#include "map_knowledge.h"
 #include "mon-util.h"
 #include "options.h"
 #include "stringutil.h"
 #include "tags.h"
 #include "terrain.h"
+#include "tiles-build-specific.h"
 #include "travel.h"
 #include "view.h"
 
@@ -34,15 +34,21 @@ static bool _mon_needs_auto_exclude(const monster* mon, bool sleepy = false)
 {
     // These include the base monster's name in their name, but we don't
     // want things in the auto_exclude option to match them.
-    if (mon->type == MONS_PILLAR_OF_SALT || mon->type == MONS_BLOCK_OF_ICE)
+    if (mon->type == MONS_PILLAR_OF_SALT || mon->type == MONS_BLOCK_OF_ICE
+        || mons_class_is_test(mon->type)) // don't autoexclude test statues/spawners
+    {
+        return false;
+    }
+
+    // Please don't autoexclude your friends. It'll hurt their feelings.
+    if (mon->wont_attack())
         return false;
 
     if (mon->is_stationary())
         return !sleepy;
 
-    // Auto exclusion only makes sense if the monster is still asleep or if it
-    // is lurking (discovered mimics).
-    return mon->asleep() || mons_is_lurking(mon);
+    // Auto exclusion only makes sense if the monster is still asleep.
+    return mon->asleep();
 }
 
 // Check whether a given monster is listed in the auto_exclude option.
@@ -57,8 +63,7 @@ static bool _need_auto_exclude(const monster* mon, bool sleepy = false)
     {
         if (pat.matches(name)
             && _mon_needs_auto_exclude(mon, sleepy)
-            && (mon->attitude == ATT_HOSTILE
-                || mon->type == MONS_HYPERACTIVE_BALLISTOMYCETE))
+            && (mon->attitude == ATT_HOSTILE))
         {
             return true;
         }
@@ -69,103 +74,71 @@ static bool _need_auto_exclude(const monster* mon, bool sleepy = false)
 
 // Nightstalker reduces LOS, so reducing the maximum exclusion radius
 // only makes sense. This is only possible because it's a permanent
-// mutation; the lantern of Shadows should not have this effect.
-// TODO: update the radiuses on wield/unwield, we already need to do that
-// when gaining/losing nightstalker.
+// mutation; other sources of LOS reduction should not have this effect.
+// Similarly, Barachim's LOS increase.
 static int _get_full_exclusion_radius()
 {
-    return LOS_RADIUS - player_mutation_level(MUT_NIGHTSTALKER);
+    // XXX: dedup with update_vision_range()!
+    return LOS_DEFAULT_RANGE - you.get_mutation_level(MUT_NIGHTSTALKER)
+                             + (you.species == SP_BARACHI ? 1 : 0);
 }
 
-// If the monster is in the auto_exclude list, automatically set an
-// exclusion.
-void set_auto_exclude(const monster* mon)
+/**
+ * Adds auto-exclusions for any monsters in LOS that need them.
+ */
+void add_auto_excludes()
 {
-    if (!is_map_persistent())
+    if (!is_map_persistent() || !map_bounds(you.pos()))
         return;
 
-    // Something of a speed hack, but some vaults have a TON of plants.
-    if (mon->type == MONS_PLANT)
+    vector<monster*> mons;
+    for (radius_iterator ri(you.pos(), LOS_DEFAULT); ri; ++ri)
+    {
+        monster *mon = monster_at(*ri);
+        if (!mon || mon->is_summoned())
+            continue;
+        // Something of a speed hack, but some vaults have a TON of plants.
+        if (mon->type == MONS_PLANT)
+            continue;
+        if (_need_auto_exclude(mon) && !is_exclude_root(*ri))
+        {
+            int radius = _get_full_exclusion_radius();
+            set_exclude(*ri, radius, true);
+            mons.emplace_back(mon);
+        }
+    }
+
+    if (mons.empty())
         return;
 
-    if (_need_auto_exclude(mon) && !is_exclude_root(mon->pos()))
-    {
-        int rad = _get_full_exclusion_radius();
-        if (mon->type == MONS_HYPERACTIVE_BALLISTOMYCETE)
-            rad = 2;
-        set_exclude(mon->pos(), rad, true);
-        // FIXME: If this happens for several monsters in the same turn
-        //        (as is possible for some vaults), this could be really
-        //        annoying. (jpeg)
-        mprf(MSGCH_WARN,
-             "Marking area around %s as unsafe for travelling.",
-             mon->name(DESC_THE).c_str());
-
-#ifdef USE_TILE
-        viewwindow();
-#endif
-        learned_something_new(HINT_AUTO_EXCLUSION, mon->pos());
-    }
+    mprf(MSGCH_WARN, "Marking area around %s as unsafe for travelling.",
+            describe_monsters_condensed(mons).c_str());
+    learned_something_new(HINT_AUTO_EXCLUSION);
 }
 
-// Clear auto exclusion if the monster is killed or wakes up with the
-// player in sight. If sleepy is true, stationary monsters are ignored.
-void remove_auto_exclude(const monster* mon, bool sleepy)
-{
-    if (_need_auto_exclude(mon, sleepy))
-    {
-        del_exclude(mon->pos());
-#ifdef USE_TILE
-        viewwindow();
-#endif
-    }
-}
-
-static opacity_type _feat_opacity(dungeon_feature_type feat)
-{
-    return feat_is_opaque(feat) ? OPC_OPAQUE
-         : feat_is_tree(feat)   ? OPC_HALF : OPC_CLEAR;
-}
-
-// A cell is considered clear unless the player knows it's
-// opaque.
-class opacity_excl : public opacity_func
-{
-public:
-    CLONE(opacity_excl)
-
-    opacity_type operator()(const coord_def& p) const
-    {
-        map_cell& cell = env.map_knowledge(p);
-        if (!cell.seen())
-            return OPC_CLEAR;
-        else if (!cell.changed())
-            return _feat_opacity(env.grid(p));
-        else if (cell.feat() != DNGN_UNSEEN)
-            return _feat_opacity(cell.feat());
-        else
-            return OPC_CLEAR;
-    }
-};
-static opacity_excl opc_excl;
-
-// Note: circle_def(r, C_ROUND) gives a circle with square radius r*r+1;
-// this doesn't work well for radius 0, but then we want to
-// skip LOS calculation in that case anyway since it doesn't
-// currently short-cut for small bounds. So radius 0, 1 are special-cased.
 travel_exclude::travel_exclude(const coord_def &p, int r,
                                bool autoexcl, string dsc, bool vaultexcl)
     : pos(p), radius(r),
-      los(los_def(p, opc_excl, circle_def(r, C_ROUND))),
       uptodate(false), autoex(autoexcl), desc(dsc), vault(vaultexcl)
 {
+    const monster* m = monster_at(p);
+    if (m)
+    {
+        // Don't exclude past glass for stationary monsters.
+        if (m->is_stationary())
+            los = los_def(p, opc_fully_no_trans, circle_def(r, C_SQUARE));
+        else
+            los = los_def(p, opc_excl, circle_def(r, C_SQUARE));
+    }
+    else
+        los = los_def(p, opc_excl, circle_def(r, C_SQUARE));
     set_los();
 }
 
 // For exclude_map[p] = foo;
 travel_exclude::travel_exclude()
     : pos(-1, -1), radius(-1),
-      los(coord_def(-1, -1), opc_excl, circle_def(-1, C_ROUND)),
+      los(coord_def(-1, -1), opc_excl, circle_def(-1, C_SQUARE)),
       uptodate(false), autoex(false), desc(""), vault(false)
 {
 }
@@ -178,7 +151,7 @@ void travel_exclude::set_los()
     if (radius > 1)
     {
         // Radius might have been changed, and this is cheap.
-        los.set_bounds(circle_def(radius, C_ROUND));
+        los.set_bounds(circle_def(radius, C_SQUARE));
         los.update();
     }
 }
@@ -257,7 +230,7 @@ void exclude_set::add_exclude_points(travel_exclude& ex)
     else
         ex.los.update();
 
-    for (radius_iterator ri(ex.pos, ex.radius, C_ROUND); ri; ++ri)
+    for (radius_iterator ri(ex.pos, ex.radius, C_SQUARE); ri; ++ri)
         if (ex.affects(*ri))
             exclude_points.insert(*ri);
 }
@@ -402,8 +375,7 @@ static void _tile_exclude_gmap_update(const coord_def &p)
         for (int y = -8; y <= 8; y++)
         {
             const coord_def pc(p.x + x, p.y + y);
-            if (in_bounds(pc))
-                tiles.update_minimap(pc);
+            tiles.update_minimap(pc);
         }
 }
 #endif
@@ -418,6 +390,8 @@ static void _exclude_update(const coord_def &p)
 {
 #ifdef USE_TILE
     _tile_exclude_gmap_update(p);
+#else
+    UNUSED(p);
 #endif
     _exclude_update();
 }
@@ -440,7 +414,7 @@ void clear_excludes()
 {
 #ifdef USE_TILE
     // Tiles needs to update the minimap for each exclusion that is removed,
-    // but the exclusions need to be removed first.  Therefore, make a copy
+    // but the exclusions need to be removed first. Therefore, make a copy
     // of the current set of exclusions and iterate through the copy below.
     exclude_set excludes = curr_excludes;
 #endif
@@ -459,7 +433,7 @@ void clear_excludes()
 static void _exclude_gate(const coord_def &p, bool del = false)
 {
     set<coord_def> all_doors;
-    find_connected_identical(p, all_doors);
+    find_connected_identical(p, all_doors, true);
     for (const auto &dc : all_doors)
     {
         if (del)
@@ -475,7 +449,7 @@ void cycle_exclude_radius(const coord_def &p)
 {
     if (travel_exclude *exc = curr_excludes.get_exclude_root(p))
     {
-        if (feat_is_door(grd(p)))
+        if (feat_is_door(env.grid(p)) && env.map_knowledge(p).known())
         {
             _exclude_gate(p, exc->radius == 0);
             return;
@@ -510,9 +484,8 @@ void set_exclude(const coord_def &p, int radius, bool autoexcl, bool vaultexcl,
     {
         if (exc->desc.empty() && defer_updates)
         {
-            const int cl = env.cgrid(p);
-            if (env.cgrid(p) != EMPTY_CLOUD)
-                exc->desc = cloud_name_at_index(cl) + " cloud";
+            if (cloud_struct* cloud = cloud_at(p))
+                exc->desc = cloud->cloud_name(true) + " cloud";
         }
         else if (exc->radius == radius)
             return;
@@ -551,12 +524,8 @@ void set_exclude(const coord_def &p, int radius, bool autoexcl, bool vaultexcl,
                 }
             }
         }
-        else
-        {
-            const int cl = env.cgrid(p);
-            if (env.cgrid(p) != EMPTY_CLOUD)
-                desc = cloud_name_at_index(cl) + " cloud";
-        }
+        else if (cloud_struct* cloud = cloud_at(p))
+            desc = cloud->cloud_name(true) + " cloud";
 
         curr_excludes.add_exclude(p, radius, autoexcl, desc, vaultexcl);
     }
@@ -566,7 +535,8 @@ void set_exclude(const coord_def &p, int radius, bool autoexcl, bool vaultexcl,
 }
 
 // If a cell that was placed automatically no longer contains the original
-// monster (or it is invisible), remove the exclusion.
+// monster (or it is invisible), or if the player is no longer vulnerable to a
+// damaging cloud, then remove the exclusion.
 void maybe_remove_autoexclusion(const coord_def &p)
 {
     if (travel_exclude *exc = curr_excludes.get_exclude_root(p))
@@ -575,14 +545,19 @@ void maybe_remove_autoexclusion(const coord_def &p)
             return;
 
         const monster* m = monster_at(p);
-        if (!m || !you.can_see(m)
-            || m->attitude != ATT_HOSTILE
-                && m->type != MONS_HYPERACTIVE_BALLISTOMYCETE
-            || strcmp(mons_type_name(m->type, DESC_PLAIN).c_str(),
-                      exc->desc.c_str()) != 0)
+        // We don't want to remove excluded clouds, check exc desc
+        // XXX: This conditional is a mess.
+        string desc = exc->desc;
+        bool cloudy_exc = ends_with(desc, "cloud");
+        if ((!m || !you.can_see(*m)
+                || !_need_auto_exclude(m)
+                || mons_type_name(m->type, DESC_PLAIN) != desc)
+            && !cloudy_exc)
         {
             del_exclude(p);
         }
+        else if (cloudy_exc && you.cloud_immune())
+            del_exclude(p);
     }
 }
 
@@ -605,7 +580,7 @@ string exclude_set::get_exclusion_desc()
         if (ma != auto_unique_annotations.end())
             continue;
 
-        if (ex.desc != "")
+        if (!ex.desc.empty())
             desc.push_back(ex.desc);
         else
             count_other++;
@@ -691,6 +666,7 @@ void marshallExcludes(writer& outf, const exclude_set& excludes)
 
 void unmarshallExcludes(reader& inf, int minorVersion, exclude_set &excludes)
 {
+    UNUSED(minorVersion);
     excludes.clear();
     int nexcludes = unmarshallShort(inf);
     if (nexcludes)
